#ifndef __CCA_H__
#define __CCA_H__

#include <string>

/**  The CCA (and the Sandia developers of CCA codes) no longer hold
 * with the following view, but the macro remains defined because
 * some codes still depend on it.
 * Use of the C/C++ const modifier at compile time is not allowed in
 *    the standard, as it is unenforceable. It is nonetheless a nice
 *    mnemonic to assist communication among scientific
 *    programmers. 
 */
#define CONST
/** Some compilers are finicky about whether namespaces are closed
    with a "}" or a "};"
    If a compiler is not picky, then "} " is preferred.
 */
#define ENDSEMI   // use of this macro is deprecated
#define CCA_NAMESPACE_ENDSEMICOLON 

/** CFREE/CDELETE indicates returned pointers are owned by the caller and should
    be free()'d or deleted at the earliest appropriate time. 
    These hints will be going away, with the introduction of a memory
    management model in the neoclassic cca specification for c++.
*/
#define CFREE
#define CDELETE

/** NO_PURE_VIRTUAL can be defined by adding --disable-pure in configuration 
  step, it will genetare spec-classic without pure virtual classes
*/
#ifdef NO_PURE_VIRTUAL
#define CLASSIC_CCA_PURE
#else
#define CLASSIC_CCA_PURE = 0 
#endif //  NO_PURE_VIRTUAL

/** To deal with babel taking over this same namespace we name this
one "classic".

NOTES: 

<p>We have to have a sane memory management model, 
standardized one and lighter weight than the
SIDL/BABEL system. Therefore, cca-spec-neo exists. See it.
</p>
<p>
The cca v-0.5 draft c++ spec interfaces will continue to exist as
defined, but their use is deprecated.
</p>
 */
namespace classic {

namespace gov {
/** 
 * <p>The spec lives in C++ namespace ::classic::gov::cca .</p>
 */
  namespace cca {

    /** UNADOPTED: A class to support the runtime introspection of the CCA version.
        The int returning functions are capitalized because sysmacros.h
        #defines both major() and minor(), sheesh. */
    class Version {
    public:
      /** The descriptive source release string. */
      static CONST char *name(); 
      /** The dot notation release number string; major.minor.patch.  */
      static CONST char *id();
      /**@ Revision numbering scheme will be rigorously followed after
           1_0_0. */
      //@{
      /** Major version; increments with binary incompatible changes 
          in this header. */
      static CONST int Major(); 
      /** Minor revision; increments with binary incompatible changes in
          well known ports, including changes in namings expressed with
          string name or type arguments. Changes in the spec but outside
          this header. */
      static CONST int Minor(); 
      /** Patch revision; binary compatible changes, such as addition of
          new common port definitions. */
      static CONST int Patch(); 
      //@}
    };

 
//===================================================================

    /** A tag interface to identify an interface capable of being exported to
	or imported from a CCA component. */
    class Port  {
    public:
      /** obligatory virtual destructor */
      virtual ~Port (){}
    };

//===================================================================
    /** An interface (deprecated) that describes a Port. */
    class PortInfo  {
    public:
      /** obligatory virtual destructor */
      virtual ~PortInfo(){}

      /** Returns the type for this Port.  This string corresponds to the 
	  interface name of the port, but may not be identical. */ 
      virtual CONST char* getType() CONST CLASSIC_CCA_PURE;

      /** Returns the instance name for this Port.
          This name must be unique within the scope of 
          the CCA component which uses or provides this port.*/
      virtual CONST char* getName() CONST CLASSIC_CCA_PURE;

      /** Returns the string value for this property,
          or 0 if it is undefined. */
      virtual CONST char* getProperty(char* name) CLASSIC_CCA_PURE;

    };

//===================================================================
    /** An opaque reference to a Component. */
    class ComponentID  {
    public:
      /** obligatory virtual destructor */
      virtual ~ComponentID() {}

      /** Produce a string that, within the current
          framework, that uniquely defines this component reference.
         @return toString is probably a bad choice of name for this
            method as toString is likely to be used in a more
            Java-conventional sense. */
      virtual CFREE char* toString() CLASSIC_CCA_PURE;


    };

//===================================================================
    /** The handle through which 
     * the component communicates with its
     *  containing framework.
     */
    class Services  {
    public:
      /** obligatory virtual destructor */
      virtual ~Services (){}

      /** Creates a PortInfo to be used in subsequent
          calls to describe a Port. The arguments given are copied, not kept.
          properties is an even length list (argv) of key/value pairs 
          terminated by a NULL key. Note that names of Ports in a 
          component are not distinct for used and provided ports. 
          Standard properties: (UNADOPTED, extensible list)
                MIN_CONNECTIONS (int >= 0; default 0)
		MAX_CONNECTIONS (int >= 1, default 1)
		ABLE_TO_PROXY   (true,false; default false)
      */
      virtual PortInfo *  createPortInfo(CONST char *name, CONST char *type, CONST char** properties) CLASSIC_CCA_PURE;

      /**  Fetch a port from the framework. Normally this means a uses port.
      If no uses port is connected and a provided port of the name requested
      is available, the component will be handed back that port.
       Returns NULL if no provided port or uses port connection matches name.
       @see Services.java; UNADOPTED C++ definition of "an error occurs".
   */
      virtual Port *getPort(CONST char *name) CLASSIC_CCA_PURE;
      /** stl version of getport */
      virtual Port *getPort(const std::string & name) CLASSIC_CCA_PURE;

      /** Free's the port indicated by the instance name for modification
	  by the component's containing framework.  After this call the
	  port will not be valid for use until getPort() is called
	  again.*/
      virtual void releasePort(CONST char *name) CLASSIC_CCA_PURE;
      /** stl version of releaseport */
      virtual void releasePort(const std::string & name) CLASSIC_CCA_PURE;

      /** Notifies the framework that a port described by PortInfo
	  may be used by this component. The portinfo is obtained 
      from createPortInfo. Returns nonzero if there is an error
      in registering, such as bad PortInfo or already registered.
       @see Services.java; UNADOPTED C++ definition of "an error occurs".
      */
      virtual int registerUsesPort(PortInfo *name_and_type) CLASSIC_CCA_PURE;

      /** always returns 0. If it fails, throws a std::exception or
       * something derived therefrom. No properties option is
       * allowed. */
      virtual int registerUsesPort( const std::string & portName, const std::string & portType) CLASSIC_CCA_PURE;

      /** Notify the framework that a Port, previously
          registered by this component, is no longer desired.
          Returns nonzero if the port is still in use, ignoring
          the unregister request.
       @see Services.java; UNADOPTED C++ definition of "an error occurs".
       */
      virtual int unregisterUsesPort(CONST char *name) CLASSIC_CCA_PURE;
      /** always returns 0. if an error occurs, throws something derived
       * from std::exception. */
      virtual int unregisterUsesPort(const std::string & name) CLASSIC_CCA_PURE;

      /** Exports a Port implemented by this component to the framework.  
	  This Port is now available for the framework to
	  connect to other components. The PortInfo is obtained 
      from createPortInfo. Returns nonzero if addProvidesPort fails,
      for example, because that name is already provided.
       @see Services.java; UNADOPTED C++ definition of "an error occurs". */
      virtual int addProvidesPort(Port *inPort, PortInfo *name) CLASSIC_CCA_PURE;

      /** always returns 0. If it fails, throws a std::exception or
       * something derived therefrom. No properties option is
       * allowed. */
      virtual int addProvidesPort(Port *inPort, const std::string & portName, const std::string & portType) CLASSIC_CCA_PURE;

      /** Notifies the framework that a previously exported Port is no longer 
	  available for use. */
      virtual void removeProvidesPort(CONST char *name) CLASSIC_CCA_PURE;

      /** always returns 0. if an error occurs, throws something derived
       * from std::exception. */
      virtual void removeProvidesPort(const std::string & name) CLASSIC_CCA_PURE;

      /** Get a reference to the component which this 
          Services object belongs. Delete it when you're done
       with it.
       */
      virtual CDELETE ComponentID* getComponentID() CLASSIC_CCA_PURE;

    };

//===================================================================
    /** Component object model 
     * that all Components must implement.
     */
    class Component {
    public:
      /** obligatory virtual destructor */
      virtual ~Component(){}

      /** The components containing framework provides services through
	  the Services interface. */
      virtual void setServices(Services *cc) CLASSIC_CCA_PURE;
    };

//===================================================================

} CCA_NAMESPACE_ENDSEMICOLON // end namespace cca
} CCA_NAMESPACE_ENDSEMICOLON // end namespace gov
} CCA_NAMESPACE_ENDSEMICOLON // end namespace classic

#endif //__CCA_H_ 
